% Exploratory Analysis of 2007 CIA Factbook Data Using Self Organizing Maps 
% KOM Group 
% Bentley College
% Sep. 16, 2008 

% Make the data
sD=som_read_data('D:\CSBIGS\Vol2-2Leads\MariaGuangyingKohonen\data.dat','x')

%Normalize the data
sD=som_normalize(sD,'log')
sD=som_normalize(sD,'var')

% Make the SOM
sM=som_make(sD, 'msize', [9 12], 'lattice', 'hexa');
% seq training algorithm
% sM=som_make(sD, 'algorithm','seq');

sMlab=som_autolabel(sM,sD)
U=som_umat(sM);
Um=U(1:2:size(U,1),1:2:size(U,2));

% Basic visulization

%Figure(1)--D-matrix (grayscale)
figure(1)
h=som_cplane([sM.topol.lattice,'U'],sM.topol.msize, U(:)); 
h=som_cplane(sM, Um(:));
set(h,'Edgecolor','none'); title('D-matrix (grayscale)')

%Figure(2)--Color  Coding + Distance Matrix
figure(2)
C = som_colorcode(sM);
som_cplane(sM,C,1-Um(:)/max(Um(:)));
title('Color Coding + Distance Matrix')

%Figure(3)--Davies-Bouldin index and K-means clusters
%Figure(3) shows PCA and initial clusters
figure(3)
[Pd,V,me,l] = pcaproj(sD,2); 
Pm = pcaproj(sM,V,me); % PC-projection
Code = som_colorcode(Pm); % color coding
hits = som_hits(sM,sD);  % hits
U = som_umat(sM); % U-matrix
Dm = U(1:2:size(U,1),1:2:size(U,2)); % distance matrix
Dm = 1-Dm(:)/max(Dm(:)); 
Dm(find(hits==0)) = 0; % clustering info

subplot(1,3,1)
[c,p,err,ind] = kmeans_clusters(sM, 7); % find at most 7 clusters
plot(1:length(ind),ind,'x-')
[dummy,i] = min(ind)
cl = p{i};

subplot(1,3,2)
som_cplane(sM,Code,Dm)

subplot(1,3,3)
som_cplane(sM,cl)

%Figure(4) shows PCA and initial clusters
figure(4)
som_grid(sM,'Coord',Pm,'MarkerColor',Code,'Linecolor','k');
hold on, plot(Pd(:,1),Pd(:,2),'k+'), hold off, axis tight, axis equal
title('PC projection')

%Figure(5)--U-matrix
figure(5)
som_show(sMlab,'umat','all')
som_show_add('label',sMlab,'Textsize',12)

%Figure(6)--Complane
figure(6)
som_show(sM,'comp','all')

U=som_umat(sM);
Um=U(1:2:size(U,1),1:2:size(U,2));


% Qe and Te
[qe,te]=som_quality(sM,sD)
